"""
 Security-related hotfix for:
 
 ZClass changeClassId permissions problem.
 ZClass getClassAttr permissions problem.
 ZClass setClassAttr permissions problem.
 ZClass delClassAttr permissions problem.
 PropertyManager propertyMap mutability problem.
 PropertySheet hasProperty mutability problem.
 PropertySheet propertyIds mutability problem.
 PropertySheet propertyValues mutability problem.
 PropertySheet propertyItems mutability problem.
 PropertySheet propertyInfo mutability problem.
 PropertySheet propertyMap mutability problem.
 ObjectManager objectMap mutability problem.
 ObjectManager objectMap_d mutability problem.
"""

if 1:
    # Add ZClass permission declarations.
    from ZClasses.ZClass import ZClass
    ZClass.changeClassId__roles__ = ()  # Private
    ZClass.getClassAttr__roles__ = ()  # Private
    ZClass.setClassAttr__roles__ = ()  # Private
    ZClass.delClassAttr__roles__ = ()  # Private



if 1:
    # Fix OFS.PropertyManager.
    from OFS.PropertyManager import PropertyManager

    def _propertyMap(self):
        """Return a tuple of mappings, giving meta-data for properties """
        return self._properties

    def propertyMap(self):
        """
        Return a tuple of mappings, giving meta-data for properties.
        Return copies of the real definitions for security.
        """
        return map(lambda dict: dict.copy(), self._propertyMap())

    PropertyManager._propertyMap = _propertyMap
    PropertyManager.propertyMap = propertyMap



if 1:
    # Fix OFS.PropertySheets.
    from OFS.PropertySheets import PropertySheet, DAVProperties, FixedSchema
    
    def hasProperty(self, id):
        # Return a true value if a property exists with the given id.
        for prop in self._propertyMap():
            if id==prop['id']:
                return 1
        return 0

    def propertyIds(self):
        # Return a list of property ids.
        return map(lambda i: i['id'], self._propertyMap())

    def propertyValues(self):
        # Return a list of property values.
        return map(lambda i, s=self: s.getProperty(i['id']),
                   self._propertyMap())

    def propertyItems(self):
        # Return a list of (id, property) tuples.
        return map(lambda i, s=self: (i['id'], s.getProperty(i['id'])), 
                   self._propertyMap())

    def propertyInfo(self, id):
        # Return a mapping containing property meta-data
        for p in self._propertyMap():
            if p['id']==id: return p
        raise ValueError, 'The property %s does not exist.' % id

    def _propertyMap(self):
        # Return a tuple of mappings, giving meta-data for properties.
        # Some ZClass instances dont seem to have an _properties, so
        # we have to fake it...
        return self.p_self()._properties

    def propertyMap(self):
        # Returns a secure copy of the property definitions.
        return map(lambda dict: dict.copy(), self._propertyMap())

    def _propdict(self):
        dict={}
        for p in self._propertyMap():
            dict[p['id']]=p
        return dict

    PropertySheet.hasProperty = hasProperty
    PropertySheet.propertyIds = propertyIds
    PropertySheet.propertyValues = propertyValues
    PropertySheet.propertyItems = propertyItems
    PropertySheet.propertyInfo = propertyInfo
    PropertySheet._propertyMap = _propertyMap
    PropertySheet.propertyMap = propertyMap
    PropertySheet._propdict = _propdict
    
    def _propertyMap(self):
        return self.pm
    
    def propertyMap(self):
        return map(lambda dict: dict.copy(), self._propertyMap())
    
    DAVProperties._propertyMap = _propertyMap
    DAVProperties.propertyMap = propertyMap

    def _propertyMap(self):
        # Return a tuple of mappings, giving meta-data for properties.
        r = []
        for d in self._base._propertyMap():
            d = d.copy()
            mode = d.get('mode', 'wd')
            if 'd' in mode:
                d['mode']=filter(lambda c: c != 'd', mode)
            r.append(d)
            
        return tuple(r)

    propertyMap = _propertyMap

    FixedSchema.propertyMap = propertyMap
    FixedSchema._propertyMap = _propertyMap



if 1:
    # Fix ObjectManager.
    from OFS.ObjectManager import ObjectManager
    
    def objectMap(self):
        # Return a tuple of mappings containing subobject meta-data
        return map(lambda dict: dict.copy(), self._objects)

    def objectMap_d(self,t=None):
        if hasattr(self, '_reserved_names'): n=self._reserved_names
        else: n=()
        if not n: return self._objects
        r=[]
        a=r.append
        for d in self._objects:
            if d['id'] not in n: a(d.copy())
        return r

    ObjectManager.objectMap = objectMap
    ObjectManager.objectMap_d = objectMap_d


